#!/usr/bin/python
# -*- coding: utf_8 -*-

#
#   Copyright (C) 2009 Sofía Martin, Fernando López, LINTI, UNLP.
#   Copyright (C) 2007 Canonical Ltd.
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2, or (at your option)
#   any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

import gtk
import gtk.glade
import subprocess
from ip import *
from configuraciones import *
from IniParser import *

class DHCPConfig(dict):
	"""DHCPConfig es un diccionario que al ser instanciado carga un archivo de configuración
de DHCPD"""
	def __init__(self, archivo):
		"Recibe como argumento el path de un archivo de configuración de DHCPD"
		super(DHCPConfig, self).__init__()
		items = ['range','domain-name','domain-name-servers','broadcast-address','routers','subnet-mask','filename','root-path']

		for key in items[1:]:
			self[key] = ''
		for key in ('range_from', 'range_to'):
			self[key] = ''
		self.obligatorios = ('range_from', 'range_to', 'subnet-mask', 'filename', 'root-path')
		try:
			file = open(archivo, 'r')
		except:
			print 'No se puede acceder al archivo'
			return
		for line in file.read().split('\n'):
			line = line.strip().lstrip('option ')
			for item in items:
				if line.startswith(item+' '):
					if item == 'range':
						range_from = line.strip('range').strip(';').split()[0]
						range_to = line.strip('range').strip(';').split()[1]
						self['range_from'] = range_from
						self['range_to'] = range_to
					else:						
						self[item] = line.strip(item).strip(';').split()[0].strip('"')
	def tiene_todos_los_campos(self):
		"Retorna True si el archivo tiene configurados todos los campos definidos como obligatorios"
		ok = True
		for key in self.obligatorios:
			if not self[key]:
				# Retorna false si el valor del dict es un string vacío
				return False
		return ok

class DHCPManager:
	"""Clase para manejar el formulario de configuración de DHCP, las responsabilidades
de esta instancia van desde llenar los campos con la configuración existente
hasta validar los campos."""
	def __init__(self, gladeUI):
		self.boxipdesde = gladeUI.get_widget("ipdesde")
		self.boxiphasta = gladeUI.get_widget("iphasta")		
		self.boxdominio = gladeUI.get_widget("dominio")
		self.boxdnsserver = gladeUI.get_widget("dnsserver")				
		self.comboxinterfaces = gladeUI.get_widget("interfaces")
		self.cargar_campos()


	def campos_no_vacios(self):
		"Verifica que los campos obligatorios no estén vacíos."
		if (self.boxipdesde.get_text() == '') or (self.boxiphasta.get_text() == '') :
			error('Se deben llenar los campos obligatorios')
			return False
		return True

	def validar_campos(self):
		"""Comprobación entre la ip, máscara, rango, broadcast"""
		ip = mask = desde = hasta = dnsserver = 0
		# Obtenemos la interfaz seleccionada e instanciamos un objeto que la representa
		active = self.comboxinterfaces.get_active()
		modelo = self.comboxinterfaces.get_model()
		self.interfaz = Interfaz(modelo[active][0])
		if not self.interfaz.obtener_ip():
			raise IPException('La interfaz no tiene una ip estática configurada')
		
		# Convertimos las IP y la máscara de notación decimal a enteros de 32bits para
		# poder operar con ellas.
		ip = stringToIp(self.interfaz.obtener_ip())
		desde = stringToIp(self.boxipdesde.get_text())
		hasta = stringToIp(self.boxiphasta.get_text())
		mask = stringToIp(self.interfaz.obtener_mascara())
		net = getNet(ip, mask)
		self.net = ipToString(net)
		self.broadcast = ipToString(netBroadCast(net, mask))
		
		# El DNS no se valida por el momento así que no es necesario obtenerlo
		#if (self.boxdnsserver.get_text() != ''):
		#	dnsserver = stringToIp(self.boxdnsserver.get_text())
		
		if not (ipsAreInTheSameNet(desde, hasta, mask)) or not(ipsAreInTheSameNet(desde, ip, mask)):				
			raise IPException('El rango de IP no está bien configurado')

		return True
		
	def guardar_datos(self,ssh):
		"Guarda los datos ingresados en la configuración de DHCPD"
		if self.campos_no_vacios() and self.validar_campos():
			if not self.configurar_iptables():
				return False
			# se guarda el resultado de cada uno de los métodos, ya que si lo hacemos en el return al evaluar 
			# de izq a derecha, al obtener el primer True, retorna ese valor y no ejecuta los demás métodos,
			# los cuales no sólo devuelven True o False sino que además ejecutan algunas acciones necesarias
			result_xdmcp = self.configurar_xdmcp(ssh) 
			result_dhcpd = self.configurar_dhcpd()
			result_nfs = self.configurar_nfs()
			result_default_int_dhcp = self.configurar_default_int_dhcp()
			result_tftp = self.corregir_tftp()
			return result_dhcpd or result_nfs or result_default_int_dhcp or result_tftp or result_xdmcp
		else:
			return False
	
	def configurar_xdmcp(self,ssh):
		"Habilita a xdmcp en el archivo de configuración /etc/gdm/gdm.conf en caso que no esté habilitado"
		archivo = MinIniParser()
		archivo.readfp(open(GDM_CONF))
		#ssh es true si está habilitado, por lo cual vamos a sacar xdmcp del archivo gdm.conf
		try:
			archivo.set('xdmcp','Enable',str(not ssh))
		except ConfigParser.NoSectionError:
				archivo.add_section('xdmcp')
				archivo.set('xdmcp','Enable',str(not ssh))	
		f=open(TMP_GDM_CONF,'w')	
		archivo.write(f)
		f.close()
		return guardar_cambios_en_archivos(GDM_CONF,TMP_GDM_CONF)
			
	def configurar_iptables(self):
		if hacer_backup(IPTABLES_ACTIVE):	
			if (exists(IPTABLES)):
				subprocess.call([IPTABLES_SCRIPT, self.interfaz.obtener_nombre_interfaz(),IPTABLES_ACTIVE ])
			else:
				dialog = gtk.MessageDialog(type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_YES_NO, message_format='Puede ser que algunos archivos necesarios para configurar iptables no estén disponibles,\n ¿Desea que esta herramienta realice la configuración por usted?')
				dialog.set_title('Personalización de la configuración de iptables')
				response = dialog.run()
				dialog.destroy()
				if response == gtk.RESPONSE_YES:
					subprocess.call([IPTABLES_SCRIPT, self.interfaz.obtener_nombre_interfaz(),DEBIAN_IPTABLES_CONF])			
		return True
		
	def corregir_tftp(self):
		"""Verifica si el servicio tftp está habilitado en el archivo de configuración /etc/inetd.conf, puede ser que se haya borrado la configuración
		en caso que se desinstale el programa"""
		f = open(CONF_SUPER_SERVER, 'r')
		empieza = False
		for line in f:
			if line.lstrip().startswith('tftp'):
				empieza=True
				break
		f.close()
		if not empieza and not subprocess.call('dpkg' + ' -l' + ' tftpd-hpa', shell=True) and os.path.exists(CONF_SUPER_SERVER) and os.path.exists(CONF_POSTINST_TFTP):
			os.system(CONF_POSTINST_TFTP + ' configure')
			return True
		return False
		
	def cargar_campos(self):
		"""Carga los campos, si están configurados, del archivo de configuración de dhcp
		/etc/dhcp3/dhcpd.conf"""
		self.opciones_dhcp = DHCPConfig(DHCPD_CONF)
		if self.opciones_dhcp.tiene_todos_los_campos():
			self.boxipdesde.set_text(self.opciones_dhcp['range_from'])
			self.boxiphasta.set_text(self.opciones_dhcp['range_to'])
			self.boxdominio.set_text(self.opciones_dhcp['domain-name'])
			self.boxdnsserver.set_text(self.opciones_dhcp['domain-name-servers'])
	
	def configurar_dhcpd(self):
		f = open(TMP_DHCPD_CONF,'w')
		f.write("""#
# Default LTSP dhcpd.conf config file.
#
authoritative;

subnet %s netmask %s {
range %s %s;
option broadcast-address %s;
next-server %s;
option subnet-mask %s;\n""" % (self.net, self.interfaz.obtener_mascara(), self.boxipdesde.get_text(), self.boxiphasta.get_text(), self.broadcast,self.interfaz.obtener_ip(),self.interfaz.obtener_mascara()))
		if (self.boxdominio.get_text() != ''):
			f.write("option domain-name \"%s\";\n" %self.boxdominio.get_text())
		if ( self.boxdnsserver.get_text() != ''):	
			f.write("option domain-name-servers %s;" %self.boxdnsserver.get_text())
		f.write("""
option root-path "/opt/ltsp/i386";
filename "/ltsp/i386/pxelinux.0";
}""")
		f.close()
		return guardar_cambios_en_archivos(DHCPD_CONF,TMP_DHCPD_CONF)
			
		
	
	def configurar_nfs(self):
		"""Agrega si no están los directorios necesarios para exportar a los clientes,
		estos son el directorio raíz ltsp y un directorio para guardar relaciones MAC->Nombre"""
		
		# Si el directorio para guardar MACs no existe hay que crearlo (es probable que se cree
		# en LtsClientes.agregar())
		if not (os.path.exists(COMPARTIDO)):
			os.mkdir(COMPARTIDO)
		oldFile = open(NFS_CONF, 'r')
		newFile = open(TMP_NFS_CONF, 'w')
		for line in oldFile:
			if not (line.lstrip().startswith('/opt/ltsp/i386') or line.lstrip().startswith('/opt/compartido')):
				newFile.write(line.lstrip())
		newFile.write("/opt/ltsp/i386     %s/%s(ro,no_root_squash,sync,no_subtree_check)\n" %(self.net,self.interfaz.obtener_mascara()))
		newFile.write("/opt/compartido     %s/%s(rw,no_root_squash,sync,no_subtree_check)\n" %(self.net,self.interfaz.obtener_mascara()))
		newFile.close()
		oldFile.close()

		return guardar_cambios_en_archivos(NFS_CONF,TMP_NFS_CONF)
		
	def configurar_default_int_dhcp(self):
		f = open(TMP_DEFAULT_INT_DHCP,'w')
		f.write("""#
INTERFACES="%s" """ %self.interfaz.obtener_nombre_interfaz())
		f.close()
		return guardar_cambios_en_archivos(DEFAULT_INT_DHCP,TMP_DEFAULT_INT_DHCP)
		
	
